describe("ScreenService", function() {

	var templateManagerMock, $rootScopeMock, screenService;
	var zhPromise = new ZHPromise(Q);
	var i18nEngine = new I18nEngine();
	var notification;

	function buildPromiseMock() {
		var mock = {
			whenSuccess: function() {},
			whenError: function() {}
		};

		spyOn(mock, 'whenSuccess').and.callThrough();
		spyOn(mock, 'whenError').and.callThrough();
		return mock;
	}

	beforeEach(function() {
		notification = {
			name: 'change',
			id: 10
		};
		$rootScopeMock = {
			$watch: function() {},
			$broadcast: function() {}
		};
		templateManagerMock = {
			onUpdate: function() {},
			container: {
				footer: false
			},
			project: {
				maintainHistoryItems: false,
				notifications: [{
					name: 'after',
					id: 1
				}, notification, {
					name: 'before',
					id: 90
				}]
			},
			updateTemplate: function() {

			},
			projectPromise: Q.when({
				zeedhi_project: {
					footer: true
				}
			})
		};
		screenService = new ScreenService(templateManagerMock, $rootScopeMock, i18nEngine, zhPromise);
	});

	it('getHistory', function() {
		var hItems = [10, 20, 30];
		templateManagerMock.historyItems = hItems;
		expect(screenService.getHistory()).toBe(hItems);
	});

	it('clearHistoryItems', function() {
		screenService.clearHistoryItems();
		expect(templateManagerMock.historyItems.length).toBe(0);
	});

	it('filterWidget', function() {
		var data = [1, 2, 3];
		var filterFn = function() {
			return {
				then: function(fn) {
					fn(data);
				}
			};
		};
		var filterWidget = {
			currentRow: []
		};
		var widget = {
			dataSource: {
				filter: filterFn
			}
		};
		var widgets = [filterWidget, widget];
		screenService.filterWidget(filterWidget, widgets);
		expect(widget.dataSource.data).toBe(data);
	});

	it('isAlertShown', function() {
		expect(screenService.isAlertShown()).toBe(false);
	});

	it('showCustomDialog ERROR', function() {
		var type = "ERROR";
		spyOn($rootScopeMock, '$broadcast').and.callThrough();
		screenService.showCustomDialog("message", type);
		expect($rootScopeMock.$broadcast).toHaveBeenCalledWith('showCustomDialog', jasmine.objectContaining({
			icon: "error"
		}));
	});

	it('showCustomDialog ALERT', function() {
		var type = "ALERT";
		spyOn($rootScopeMock, '$broadcast').and.callThrough();
		screenService.showCustomDialog("message", type);
		expect($rootScopeMock.$broadcast).toHaveBeenCalledWith('showCustomDialog', jasmine.objectContaining({
			icon: "exclamation"
		}));
	});

	it('showCustomDialog INFORMATION', function() {
		var type = "INFORMATION";
		spyOn($rootScopeMock, '$broadcast').and.callThrough();
		screenService.showCustomDialog("message", type);
		expect($rootScopeMock.$broadcast).toHaveBeenCalledWith('showCustomDialog', jasmine.objectContaining({
			icon: "information"
		}));
	});

	it('showCustomDialog CONFIRMATION', function() {
		var type = "CONFIRMATION";
		spyOn($rootScopeMock, '$broadcast').and.callThrough();
		screenService.showCustomDialog("message", type);
		expect($rootScopeMock.$broadcast).toHaveBeenCalledWith('showCustomDialog', jasmine.objectContaining({
			icon: "question"
		}));
	});

	it('showCustomDialog SUCCESS', function() {
		var type = "SUCCESS";
		spyOn($rootScopeMock, '$broadcast').and.callThrough();
		screenService.showCustomDialog("message", type);
		expect($rootScopeMock.$broadcast).toHaveBeenCalledWith('showCustomDialog', jasmine.objectContaining({
			icon: "checked"
		}));
	});

	it('buildUserData', function(){
		var descriptions = ['1','2','3'];
		screenService.buildUserData('username',descriptions, function(){});
		templateManagerMock.clickUserData();
		expect(templateManagerMock.userData.descriptions).toEqual(descriptions);
	});

	it('getNotification', function() {
		var notif = screenService.getNotification(10);
		expect(notif).toEqual(notification);
	});

	it('setNotificationHint', function() {
		screenService.setNotificationHint(10, "any changes");
		expect(notification.hint).toBe("any changes");
	});
	it('changeLoadingMessage', function() {
		screenService.changeLoadingMessage("message");
		expect(templateManagerMock.loadingMessage).toBe("message");
	});

	it('restoreDefaultLoadingMessage', function() {
		screenService.changeLoadingMessage("message");
		screenService.restoreDefaultLoadingMessage();
		expect(templateManagerMock.loadingMessage).not.toBeDefined();
	});

	it('hideLoader', function() {
		spyOn($rootScopeMock, '$broadcast').and.callThrough();
		screenService.hideLoader();
		expect($rootScopeMock.$broadcast).toHaveBeenCalledWith('request.stop');
	});

	it('showLoader', function() {
		spyOn($rootScopeMock, '$broadcast').and.callThrough();
		screenService.showLoader();
		expect($rootScopeMock.$broadcast).toHaveBeenCalledWith('request.start');
	});

	it('editMode', function() {
		spyOn($rootScopeMock, '$broadcast').and.callThrough();
		screenService.editMode();
		expect($rootScopeMock.$broadcast).toHaveBeenCalledWith('ui.editMode');
	});

	it('viewMode', function() {
		spyOn($rootScopeMock, '$broadcast').and.callThrough();
		screenService.viewMode();
		expect($rootScopeMock.$broadcast).toHaveBeenCalledWith('ui.viewMode');
	});

	it('allowMultiSelect', function() {
		spyOn($rootScopeMock, '$broadcast').and.callThrough();
		var someWidget = {};
		screenService.allowMultiSelect(someWidget);
		expect($rootScopeMock.$broadcast).toHaveBeenCalledWith('ui.allowMultiSelect', someWidget);
	});


	it('disableMultiSelect', function() {
		spyOn($rootScopeMock, '$broadcast').and.callThrough();
		var someWidget = {};
		screenService.disableMultiSelect(someWidget);
		expect($rootScopeMock.$broadcast).toHaveBeenCalledWith('ui.disableMultiSelect', someWidget);
	});


	it('deselectAll', function() {
		spyOn($rootScopeMock, '$broadcast').and.callThrough();
		var someWidget = {};
		screenService.deselectAll(someWidget);
		expect($rootScopeMock.$broadcast).toHaveBeenCalledWith('ui.deselectAll', someWidget);
	});

	it('closeFieldView', function() {
		templateManagerMock.fieldList = {};
		templateManagerMock.container.innerFooter = {};
		spyOn($rootScopeMock, '$broadcast').and.callThrough();
		screenService.closeFieldView();
		expect($rootScopeMock.$broadcast).toHaveBeenCalledWith('ui.editMode');
		expect(templateManagerMock.fieldList).toBe(undefined);
		expect(templateManagerMock.container.innerFooter).toBe(undefined);

	});

	it('startEditWidget', function() {
		var widget = {};
		screenService.startEditWidget(widget);
		expect($rootScopeMock.widgetEditing).toBe(widget);
	});

	it('stopEditWidget', function() {
		var widget = {};
		screenService.stopEditWidget(widget);
		expect($rootScopeMock.widgetEditing).not.toBeDefined();
	});

	it('isWidgetEditing', function() {
		expect(screenService.isWidgetEditing()).toBe(false);
	});

	it('hideAbout', function() {
		spyOn($rootScopeMock, '$broadcast').and.callThrough();
		screenService.hideAbout();
		expect($rootScopeMock.$broadcast).toHaveBeenCalledWith('hideAbout');
	});

	it('isAboutShown', function() {
		expect(screenService.isAboutShown()).toBe(false);
	});

	it('closeFooter', function() {
		spyOn(templateManagerMock, 'updateTemplate').and.callThrough();
		screenService.closeFooter();
		expect($rootScopeMock.footerOpen).toBe(false);
		expect(templateManagerMock.updateTemplate).toHaveBeenCalled();
	});

	it('updateTemplate', function() {
		spyOn(templateManagerMock, 'updateTemplate').and.callThrough();
		screenService.updateTemplate();
		expect(templateManagerMock.updateTemplate).toHaveBeenCalled();
	});

	it('changeFooterSelect', function() {
		spyOn(templateManagerMock, 'updateTemplate').and.callThrough();
		screenService.changeFooterSelect("teste");
		expect(templateManagerMock.container.innerFooter).toBe("teste");
	});

	it('changeFooter', function() {
		spyOn(templateManagerMock, 'updateTemplate').and.callThrough();
		screenService.changeFooter("teste");
		expect(templateManagerMock.container.footer).toBe("teste");
	});

	it("i18n should return message from i18nEngine ", function() {
		expect(Util.isString(screenService.i18n("sistema"))).toBeTruthy();
	});

	it("attribution to footer at instantiation time with projectPromise and rootscope.watch", function(done) {
		var container = {
			zeedhi_project: {
				footer: 0
			}
		};
		templateManagerMock.projectPromise = zhPromise.when(container);
		var mock = buildPromiseMock();
		mock.fn = function() {};
		spyOn(mock, 'fn');
		$rootScopeMock.$watch = function(arg1, fn) {
			mock.fn();
			fn();
		};
		screenService = new ScreenService(templateManagerMock, $rootScopeMock, i18nEngine, zhPromise);
		templateManagerMock.projectPromise.then(mock.whenSuccess, mock.whenError).then(function() {
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(mock.whenSuccess).toHaveBeenCalled();
			done();
		});
		expect(mock.fn).toHaveBeenCalled();
	});

	it("goBack called without swipes and without history items should return rejected promise ", function(done) {
		$rootScopeMock.swipes = [];
		templateManagerMock.historyItems = [];
		var mock = buildPromiseMock();
		screenService.goBack().then(mock.whenSuccess, mock.whenError).then(function() {
			expect(mock.whenError).toHaveBeenCalled();
			expect(mock.whenSuccess).not.toHaveBeenCalled();
			done();
		});
	});

	it("goBack called without swipes and with history items should return promise ", function(done) {
		$rootScopeMock.swipes = [];
		$rootScopeMock.openWindow = function() {
			return zhPromise.when();
		};
		templateManagerMock.wizard = {};
		templateManagerMock.wizard.stages = [{}, {}];
		templateManagerMock.updateTemplate = function() {
			return zhPromise.when();
		};
		templateManagerMock.historyItems = [0, 1];
		var mock = buildPromiseMock();
		screenService.goBack().then(mock.whenSuccess, mock.whenError).then(function() {
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(mock.whenSuccess).toHaveBeenCalled();
			done();
		});
	});

	it("goBack called with swipes should return promise ", function(done) {
		$rootScopeMock.swipes = [{}, {}];
		screenService.closeSwipe = function() {
			return zhPromise.when();
		};
		var mock = buildPromiseMock();
		screenService.goBack().then(mock.whenSuccess, mock.whenError).then(function() {
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(mock.whenSuccess).toHaveBeenCalled();
			done();
		});
	});

	it("openPopup should change $rootScope's _popupStack and _currentPopup properties", function(done) {
		$rootScopeMock.$broadcast = function() {
			arguments[2].resolve();
		};
		var someWidget = {
			activate: function() {}
		};
		screenService.openPopup(someWidget).then(function() {
			expect($rootScopeMock._popupStack).toContain(someWidget);
			expect($rootScopeMock._currentPopup).toBe(someWidget);
			done();
		});
	});

	it("goBack should return promise and remove from history items", function(done) {
		$rootScopeMock.swipes = [];
		$rootScopeMock.openWindow = function() {
			return zhPromise.when();
		};
		templateManagerMock.updateTemplate = function() {
			return zhPromise.when();
		};
		templateManagerMock.historyItems = [{
			name: "myContainer"
		}, {
			name: "myContainer2"
		}];
		screenService.closeSwipe = function() {
			return zhPromise.when();
		};
		var mock = buildPromiseMock();
		screenService.goBack().then(mock.whenSuccess, mock.whenError).then(function() {
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(templateManagerMock.historyItems.length).toBe(0);
			done();
		});
	});

	it("goBack should not remove history items if maintainHistoryItems is true", function(done) {
		$rootScopeMock.swipes = [];
		$rootScopeMock.openWindow = function() {
			return zhPromise.when();
		};
		templateManagerMock.updateTemplate = function() {
			return zhPromise.when();
		};
		templateManagerMock.historyItems = [{
			name: "myContainer"
		}, {
			name: "myContainer2"
		}];
		templateManagerMock.container.fileName = 'myContainer.json';
		templateManagerMock.project.maintainHistoryItems = true;
		screenService.closeSwipe = function() {
			return zhPromise.when();
		};
		var mock = buildPromiseMock();
		screenService.goBack().then(mock.whenSuccess, mock.whenError).then(function() {
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(templateManagerMock.historyItems.length).toBe(2);
			done();
		});
	});

	it("should start a download", function() {
		var fileUrl = 'mysite.com/path/file.ext';
		var expectedObject = JSON.stringify({
			click: Function,
			style: Object({ display: 'none' }),
			href: 'mysite.com/path/file.ext',
			download: 'file.ext'
		});
		var a = {
			click : function() {},
			style : {}
		};

		spyOn(screenService, 'downloadFileFromURL').and.callThrough();
		spyOn(document, 'createElement').and.callFake(function(tagName) {
			expect(tagName).toBe('a');
			return a;
		});
		spyOn(document.body, 'appendChild').and.callFake(function(obj) {
			obj = JSON.stringify(obj);
			expect(obj).toEqual(expectedObject);
		});
		spyOn(document.body, 'removeChild').and.callFake(function(obj) {
			obj = JSON.stringify(obj);
			expect(obj).toEqual(expectedObject);
		});

		screenService.downloadFileFromURL(fileUrl);
		expect(screenService.downloadFileFromURL).toHaveBeenCalledWith(fileUrl);
		expect(document.createElement).toHaveBeenCalled();
		expect(document.body.appendChild).toHaveBeenCalled();
		expect(document.body.removeChild).toHaveBeenCalled();
	});

	it("should start a download with specific file name", function() {
		var fileUrl = 'mysite.com/path/file.ext';
		var filename = 'nameTesting';
		var expectedObject = JSON.stringify({
			click: Function,
			style: Object({ display: 'none' }),
			href: 'mysite.com/path/file.ext',
			download: filename
		});
		var a = {
			click : function() {},
			style : {}
		};

		spyOn(screenService, 'downloadFileFromURL').and.callThrough();
		spyOn(document, 'createElement').and.callFake(function(tagName) {
			expect(tagName).toBe('a');
			return a;
		});
		spyOn(document.body, 'appendChild').and.callFake(function(obj) {
			obj = JSON.stringify(obj);
			expect(obj).toEqual(expectedObject);
		});
		spyOn(document.body, 'removeChild').and.callFake(function(obj) {
			obj = JSON.stringify(obj);
			expect(obj).toEqual(expectedObject);
		});

		screenService.downloadFileFromURL(fileUrl, filename);
		expect(screenService.downloadFileFromURL).toHaveBeenCalledWith(fileUrl, filename);
		expect(document.createElement).toHaveBeenCalled();
		expect(document.body.appendChild).toHaveBeenCalled();
		expect(document.body.removeChild).toHaveBeenCalled();
	});

	it('openSwipe should call update swipe tip of previous widget currentRow', function(done) {
		var currentRow = {};
		var lastActiveWidget = {
			updateSwipeTip: function() {},
			currentRow: currentRow
		};

		var widget = {
			activate: function() {},
			lastActiveWidget: lastActiveWidget
		};

		spyOn($rootScopeMock, '$broadcast').and.callFake(function(event, widget, closeAll, deferred, customClass) {
			deferred.resolve();
		});

		spyOn(lastActiveWidget, 'updateSwipeTip');

		screenService.openSwipe(widget).then(function() {
			expect(lastActiveWidget.updateSwipeTip).toHaveBeenCalledWith(currentRow);
			done();
		});

	});
});